@@ -20,6 +20,8 @@ from typing import Any, Dict, Hashable, TypeVar
 
 import yaml
 
+from pyrallis import utils
+
 logger = getLogger(__name__)
 
 
@@ -101,7 +103,7 @@ def encode_dict(obj: Mapping) -> Dict[Any, Any]:
 
 @encode.register(Enum)
 def encode_enum(obj: Enum) -> str:
-    return obj.value
+    return obj.name
 
 
 for t in [str, float, int, bool, bytes]:
@@ -117,6 +119,13 @@ encode.register(Namespace, lambda x: encode(vars(x)))
 Dataclass = TypeVar("Dataclass")
 
 
-def dump(config: Dataclass, *args, **kwargs):
+def dump(config: Dataclass, stream=None, omit_defaults: bool = False, **kwargs):
+    """
+    Dump the config file to yaml.
+    optionally omit any value that still has a default value
+    """
     config_dict = encode(config)
-    return yaml.dump(config_dict, *args, **kwargs)
+    if omit_defaults:
+        defaults_dict = encode(utils.get_defaults_dict(config))
+        config_dict = utils.remove_matching(config_dict, defaults_dict)
+    return yaml.dump(config_dict, stream, **kwargs)
